package com.servlets;

import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

public class ChartServlet extends HttpServlet
{
	private static final long serialVersionUID = 1L;
	
	private  final  String  SCRIPT       = "<script src=\"js.data/%s\"></script>";
	private  final  String  CANVAS_name  = "chart";
	private  final  int     CHART_width  = 480;
	private  final  int     CHART_height = 320;
	
	/*
	 * Флаг вида перехода на индексную страницу 'page_index_return'.
	 * -1 вызов HttpServletResponse.sendRedirect(url) - с переопределением URL страницы
	 *  0 вызов RequestDispatcher.forward; адрес URL не изменяется
	 *  1 формирование сервлетом страницы с определением последней открытой страницы radio-button  
	 */
	private  int page_index_return = -1;
	
	public ChartServlet() {}
	
	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException 
	{
		String param_chart = request.getParameter("chart");

		String  last_call = null;
		// Последний вызов
		HttpSession session = request.getSession();
		if (session != null)
			last_call = (String) session.getAttribute("LAST_CALL");

		OutputStream outStream = response.getOutputStream();
		String content = "";
		if (param_chart == null) {
			ServletContext context = request.getSession().getServletContext();
			// Перенаправление на страницу
			if (page_index_return == -1) {
				// Возврат на страницу проекта, используемую по умолчанию, с переопределением URL.
				// Обработка в браузере
				response.sendRedirect("/chart-js");
			} else if ((page_index_return == 0) && (context != null)) {
				// Возврат на страницу по умолчанию без переопределения URL
				// Обработка в контейнере
	            RequestDispatcher dispatcher = context.getRequestDispatcher("/index.html");
	            dispatcher.forward(request, response);
			} else {
				// Определение последнего вызова
				if ((last_call != null) && (param_chart == null))
					param_chart = last_call;
				content = startPage(param_chart);
				last_call = null;
			}
		} else if (param_chart.equalsIgnoreCase("bar")) {
			last_call = param_chart;
			content = createBarChartPage();
		} else if (param_chart.equalsIgnoreCase("pie")) {
			content = createPieChartPage();
			last_call = param_chart;
		} else if (param_chart.equalsIgnoreCase("line")) {
			content = createLineChartPage();
			last_call = param_chart;
		}
		
        outStream.write(content.getBytes("UTF-8"));
        outStream.flush();
        outStream.close();
        // Сохранение последнего вызова
        if (session != null)
			session.setAttribute("LAST_CALL", last_call);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private String startPage(final String param_chart)
	{
		String barChecked  = " CHECKED";
		String pieChecked  = "";
		String lineChecked = "";
		if (param_chart != null) {
			barChecked  = (param_chart.equals("bar" ) ? " CHECKED" : "");
			pieChecked  = (param_chart.equals("pie" ) ? " CHECKED" : "");
			lineChecked = (param_chart.equals("line") ? " CHECKED" : "");
		}

		String content = "";
		content += "<!DOCTYPE html PUBLIC " +
				"\"-//W3C//DTD HTML 4.01 Transitional//EN\" " +
				"\"http://www.w3.org/TR/html4/loose.dtd\"> ";
		content += "<html>";
		content +=     "<head>";
		content +=         "<meta charset=\"UTF-8\">";
		content +=         "<title>Chart.js</title>";
		content +=     "</head>";		
		content += "<body>";
		content +=     "<h3>JavaScript библиотека Chart.js</h3>";
		content +=     "<p>Выберите тип диаграммы :</p>";
		content +=     "<FORM ACTION=\"chartserver\" METHOD=\"GET\">";
		content +=        "<INPUT TYPE=\"radio\" NAME=\"chart\"";
		content +=           "VALUE=\"bar\"" + barChecked + "> Bar Chart";
		content +=        "<INPUT TYPE=\"radio\" NAME=\"chart\""; 
		content +=           "VALUE=\"pie\"" + pieChecked + "> Pie Chart";
		content +=        "<INPUT TYPE=\"radio\" NAME=\"chart\""; 
		content +=           "VALUE=\"line\"" + lineChecked + "> Line Chart";
		content +=        "<P />";
		content +=        "<INPUT TYPE=\"submit\" VALUE=\"Generate chart\">";
		content +=     "</FORM> ";
		content += "</body>";
		content += "</html>";
		return content;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private String createDataBar(final String fname)
	{
		String script = String.format(SCRIPT, fname);
		// create bar.js
		return script;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private String createDataPie(final String fname)
	{
		String script = String.format(SCRIPT, fname);
		// create pie.js
		return script;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private String createDataLine(final String fname)
	{
		String script = String.format(SCRIPT, fname);
		// create line.js
		return script;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private String createCanvasDesc(){
		String TEMP = "<div id=\"container\" "
				          + "style=\"border:1px solid #ccc;"
				          +   "width:%dpx;height:%dpx;padding:5px\">"
                     + "<canvas id=\"%s\" width=\"%d\" height=\"%d\">"
				     + "</canvas>"
				     + "</div>";
				
		return String.format(TEMP, CHART_width + 10, CHART_height + 10, CANVAS_name, CHART_width, CHART_height);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private String createCanvasContext(){
		String TEMP = "var ctx = document.getElementById(\"%s\").getContext(\"2d\");"; 
		return String.format(TEMP, CANVAS_name);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private String createBarChartPage()
	{
		String content = "";

		content += "<!DOCTYPE html PUBLIC ";
		content +=     "\"-//W3C//DTD HTML 4.01 Transitional//EN\" ";
		content +=     "\"http://www.w3.org/TR/html4/loose.dtd\"> " ;		
		content += "<html>";
		content +=   "<head>";
		content +=         "<title>Bar Chart (chart.js)</title>";
		content +=         "<meta charset=\"UTF-8\">";
		content +=         "<script src=\"js/Chart.min.js\"></script>";
		content +=         "<script src=\"js/colors.js\"></script>";
		content +=         createDataBar("bar.js");
		content +=   "</head>";		
		content +=   "<body>";
		content +=      createCanvasDesc();
		content +=     "<script>";
		content +=     "window.onload = function() {"
				+          createCanvasContext()
				+          "window.bar = new Chart(ctx, {"
				+              "type: 'bar',"
				+              "data: barChartData,"
				+              "options: {"
				+                 "responsive: true,"
				+                 "legend: {"
				+                     "position: 'bottom',"
				+                 "},"
				+                 "title: {"
				+                     "display: true,"
				+                     "text: 'Bar Chart (Chart.js)'"
				+                 "},"
				+                 "tooltips: { "
		        +                    "callbacks: { "
		        +                      "labelColor: function(tooltipItem, chart) { "
		        +                           "return { "
		        +                              "borderColor: 'rgb(255, 0, 0)', "
		        +                              "backgroundColor: 'rgb(255, 0, 0)' "
		        +                           "} "
		        +                      "} "
		        +                    "} "
		        +                 "} "
				+              "} "
				+          "}); "
				+      "};";
		content +=     "</script>";       
		content +=     "<P />";
		content +=     "<FORM ACTION=\"chartserver\" METHOD=\"GET\">";
		content +=        "<INPUT TYPE=\"submit\" VALUE=\"Назад\">";
		content +=     "</FORM> ";
		content +=   "</body>";
		content += "</html>";
		return content;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private String createPieChartPage()
	{
		String content = "";

		content += "<!DOCTYPE html PUBLIC ";
		content +=     "\"-//W3C//DTD HTML 4.01 Transitional//EN\" ";
		content +=     "\"http://www.w3.org/TR/html4/loose.dtd\"> " ;		
		content += "<html>";
		content +=   "<head>";
		content +=         "<title>Pie Chart (chart.js)</title>";
		content +=         "<meta charset=\"UTF-8\">";
		content +=         "<script src=\"js/Chart.min.js\"></script>";
		content +=         "<script src=\"js/colors.js\"></script>";
		content +=         createDataPie("pie.js");
		content +=   "</head>";		
		content +=   "<body>";
		content +=      createCanvasDesc();
		content +=     "<script>";
		content +=     "window.onload = function() {"
				+          createCanvasContext()
				+          "window.pie = new Chart(ctx, config);"
				+      "};";
		content +=     "</script>";       
		content +=     "<P />";
		content +=     "<FORM ACTION=\"chartserver\" METHOD=\"GET\">";
		content +=        "<INPUT TYPE=\"submit\" VALUE=\"Назад\">";
		content +=     "</FORM> ";
		content +=   "</body>";
		content += "</html>";
		return content;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private String createLineChartPage()
	{
		String content = "";

		content += "<!DOCTYPE html PUBLIC ";
		content +=     "\"-//W3C//DTD HTML 4.01 Transitional//EN\" ";
		content +=     "\"http://www.w3.org/TR/html4/loose.dtd\"> " ;		
		content += "<html>";
		content +=   "<head>";
		content +=         "<title>Pie Chart (chart.js)</title>";
		content +=         "<meta charset=\"UTF-8\">";
		content +=         "<script src=\"js/Chart.min.js\"></script>";
		content +=         "<script src=\"js/colors.js\"></script>";
		content +=         createDataLine("line.js");
		content +=   "</head>";		
		content +=   "<body>";
		content +=      createCanvasDesc();
		content +=     "<script>";
		content +=     "window.onload = function() {"
				+          createCanvasContext()
				+          "window.line = new Chart(ctx, config);"
				+      "};";
		content +=     "</script>";       
		content +=     "<P />";
		content +=     "<FORM ACTION=\"chartserver\" METHOD=\"GET\">";
		content +=        "<INPUT TYPE=\"submit\" VALUE=\"Назад\">";
		content +=     "</FORM> ";
		content +=   "</body>";
		content += "</html>";
		return content;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
}
